/**  This file is part of the KDE project
 *
 *  Copyright (C) 2011 Adam Pigg <adam@piggz.co.uk>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#include <QApplication>
#include <KAboutData>

#include "KexiMobileMainWindow.h"

int main(int argc, char** argv)
{
    KAboutData aboutData( "keximobile", "keximobile",
      kxi18n("Kexi Mobile"), "0.2",
      kxi18n("A mobile optimized version of Kexi from Calligra Suite"),
      KAboutData::License_LGPL,
      kxi18n("Copyright (c) 2014 Adam Pigg") );
    KCmdLineArgs::init( argc, argv, &aboutData );

    KApplication app;
    KexiMobileMainWindow foo;
    foo.show();
    return app.exec();
}
